"""
Mother Exceptions.
"""

class MotherException(Exception):

    def __init__(self, value):
        self.value= value

    def __str__(self):
        return self.value

class QueryError(MotherException):
    def __init__(self, value= ''):
        MotherException.__init__(self,
                    "Query error - %s" % value)

class ConnectionError(MotherException):
    def __init__(self, value= ''):
        MotherException.__init__(self,
                "Db connection re-established: old session is lost.")

class BrokenConnection(MotherException):
    def __init__(self, value= ''):
        MotherException.__init__(self,
                    "Broken Connection :(")

class InsertError(MotherException):
    def __init__(self, value= ""):
        MotherException.__init__(self,
                    "Cannot insert - %s" % value)

class SelectError(MotherException):
    def __init__(self, value= ""):
        MotherException.__init__(self,
                    "Cannot select - %s" % value)

class UpdateError(MotherException):
    def __init__(self, value= ""):
        MotherException.__init__(self,
                    "Cannot update - %s " % value)

class DeleteError(MotherException):
    def __init__(self, value= ""):
        MotherException.__init__(self,
                    "Cannot delete - %s" % value)

class InternalError(MotherException):
    def __init__(self, value= ""):
        MotherException.__init__(self,
                    "Internal error - %s" % value)

class WrongTypeError(MotherException):
    def __init__(self, value= ""):
        MotherException.__init__(self, str(value))
        
class InvalidFilter(MotherException):
    def __init__(self, value= ""):
        MotherException.__init__(self, str(value))
        



from sample import *
from mother.speaker import Speaker

use_session= False

# ei, don't pretend order: this is just a test field

init_mother('moconf')
session = use_session and MotherSession('TestSession') or None

Sun= ClsStars({'star_name': 'sun'}, MO_SAVE, session= session)
id_sun= Sun.getField('star_id')
print ' ## Sun inserted with id', id_sun

Sun= ClsStars({'star_id': id_sun}, session= session)
print ' ## Sun name: ', Sun.getField('star_name', autoload= True)
Sun.load()
Sun.setField('star_mass', 12)
Sun.update()
Sun.setFields({'star_mass': 13, 'star_age': 3})
Sun.update()
Sun.delete()

Sun= ClsStars({'star_name': 'sun'}, MO_SAVE, session= session)
id_sun= Sun.getField('star_id')
Sun= ClsStars({'star_id': id_sun}, session= session)
Sun.getFields(Sun.fields, autoload= True)

Alpha= ClsStars({'star_name': 'alpha centauri'}, session= session)
Alpha.insert()

star_box= MotherBox(ClsStars, filter= None, flag= MO_LOAD, session= session)
print ' ## there are %d stars in universe' % len(star_box)

star_box= MotherBox(ClsStars, filter= {'star_name': 'sun'}, flag= MO_LOAD, session= session)
print ' ## there are %d stars in universe with name sun' % len(star_box)

star_box= MotherBox(ClsStars, filter= "star_name != 'sun'", flag= MO_LOAD, session= session)
print ' ## there are %d stars in universe with name != sun' % len(star_box)

star_box= MotherBox(ClsStars, filter= "star_name != 'sun'", flag= MO_DEL, session= session)
star_box= MotherBox(ClsStars, filter= None, flag= MO_LOAD, session= session)
print ' ## there are %d stars in universe' % len(star_box)

Alpha= ClsStars({'star_name': 'alpha centauri'}, session= session)
Alpha.insert()
alpha_id= Alpha.getField('star_id')

MotherBox(ClsStars, filter= None, fields= {'star_mass': 44}, flag= MO_UP, session= session)

Alpha= ClsStars({'star_id': alpha_id}, session= session)
res= Alpha.load(fields= ['star_mass'])
print ' ## Alpha fields: ', res

Earth= Sun.insertPlanets({'planet_name': 'earth', 'planet_mass': 42})
earth_id= Earth.getField('planet_id')
print ' ## Earth inserted with id: ', earth_id

Mars= Sun.insertPlanets({'planet_name': 'mars'})

planet_box = Sun.getMultiplePlanets()
print ' ## there are %d planets on the solar system' % len(planet_box)

planet_box= Sun.getMultiplePlanets(filter= {'planet_name': 'mars'}, order = ['planet_id'])
print ' ## there are %d planets on the solar system named mars' % len(planet_box)

try:
  planet= Sun.getPlanets({'planet_name': 'earth'})
except:
  print " ## No planet on the solar system with name earth"

Sun.updateMultiplePlanets({'planet_mass': 23})
Sun.updateMultiplePlanets({'planet_mass': 77}, filter= "planet_name = 'mars'")
Sun.updateMultiplePlanets({'planet_mass': 99}, filter= {'planet_name': 'earth'})

Sun.deleteMultiplePlanets(filter= "planet_name = 'mars'")
Sun.deleteMultiplePlanets(filter= {'planet_name': 'earth'})

Earth= Sun.insertPlanets({'planet_name': 'earth', 'planet_mass': 42})
Mars= Sun.insertPlanets({'planet_name': 'mars'})

GreenPeople= Mars.assignLifes({'life_name': 'green people'}, MO_SAVE)
green_id= GreenPeople.getField('life_id')
print ' ## inserted green people with id: ', green_id

params= Mars.paramsLifes(GreenPeople)
print ' ## relation between mars and green people: ', params

Worms = Mars.assignLifes({'life_name': 'worms'}, MO_SAVE, params= {'age': 22})
params= Mars.paramsLifes(Worms.getFields())
print ' ## relation between mars and worms: ', params

mars_lifes= Mars.joinLifes(fields= ['life_name', 'life_id'])
for m in mars_lifes.getRecords(flag_obj= True):
    print ' ## On mars %s lives' % m.getField('life_name')

mars_lifes= Mars.joinLifes(fields= ['life_name'], jfilter= {'life_name': 'worms'}, filter= {'age': 22})
print ' ## there are %d lifes on mars named worms, age 22' % len(mars_lifes)

Mars.dropMultipleLifes(jfilter= {'life_name': 'worms'}, filter= {'age': 22}, flag= MO_DEL)
Mars.dropMultipleLifes(jfilter= {'life_name': 'green people'}, flag= MO_NOA)
Mars.assignLifes({'life_id': green_id})

lifes= MotherBox(ClsLifes, filter= None, flag= MO_LOAD, session = session)
print ' ## There are %d lifes on universe' % len(lifes)

marsmoons= Mars.insertMoonsInfo({'num_moons': 3})
earthmoons= Earth.insertMoonsInfo({'num_moons': 1})

box= Sun.getMultiplePlanets(jbuilder= ClsMoon, jfilter= {'num_moons': 3})
print ' ## there are %d planets on the solar system with 3 moons' % len(box)

box= Sun.getMultiplePlanets(filter= "planet_name != 'rere'", jbuilder= ClsMoon, jfilter= {'num_moons': 3})
print ' ## there are %d planets not named rere on the solar system with 3 moons' % len(box)

box= Sun.getMultiplePlanets(fields= ['planet_name'], filter= {"planet_name": 'rere'}, jbuilder= ClsMoon, jfilter= {'num_moons': 3})
print ' ## there are %d planets named rere on the solar system with 3 moons' % len(box)

box= Sun.getMultiplePlanets(jbuilder= ClsLifes, jfilter= {'life_id': green_id})
print ' ## there are %d planets on the solar system where green people lives' % len(box)

box= Sun.getMultiplePlanets(filter= "planet_name != 'rere'", jbuilder= ClsLifes, jfilter= {'life_id': green_id})
print ' ## there are %d planets not named rere on the solar system where green people lives' % len(box)

box= Sun.getMultiplePlanets(fields= ['planet_name'], filter= {"planet_name": 'rere'}, jbuilder= ClsLifes, jfilter= {'life_id': green_id})
print ' ## there are %d planets named rere on the solar system where green lives' % len(box)

if session:
    session.endSession()



import apsw
from mother.speaker import Speaker
from mother.commons import ERR_COL, INF_COL

from mother.eccez import QueryError, BrokenConnection

class _SqliteInfo:
    dbfile= None


class MotherSqlite:
    
    def __init__(self):
        Speaker.log_insane('Initializing sqlite connection (db = %s)...', 
                INF_COL(_SqliteInfo.dbfile))
        self._connect()

    def _connect(self):
        dbfile= _SqliteInfo.dbfile
        #import os
        #if not os.path.isfile(dbfile):
            #Speaker.log_int_raise('Invalid Database %s', ERR_COL(dbfile))

        try:
            self.connection= apsw.Connection(dbfile)
            self.cursor= self.connection.cursor()
        except Exception, ss:
            Speaker.log_raise('Unable to establish a connection '
                    'with the database: %s', ERR_COL(ss), 
                    BrokenConnection)

        if self.connection.getautocommit():
            self.cursor.execute('BEGIN')

    def _rollback(self):
        self.cursor.execute('ROLLBACK')
        self.cursor.execute('BEGIN')

    def _commit(self):
        self.cursor.execute('COMMIT')
        self.cursor.execute('BEGIN')

    def _close(self):
        self.connection= None

    def _lastrowid(self):
        return self.connection.last_insert_rowid()

    def _execute(self, q, d):
        d= d or {}
        try:
            self.cursor.execute(q, d)
        except Exception, ss:
            Speaker.log_raise(ERR_COL(ss), QueryError)

    def _gquery(self, q, d):
        self._execute(q, d)
        c= self.cursor
        res= []
        for rec in c:
            drec= {}
            rec_descr= c.getdescription()
            for n, field in enumerate(rec):
                drec[rec_descr[n][0]]= field
            res.append(drec)

        return res
    
    def _qquery(self, q, d):

        self._execute(q, d)

    def get_tables(self):
        qry= ("SELECT tbl_name FROM SQLITE_MASTER WHERE type='table'")
        res= self._gquery(qry, {})
        return [d['tbl_name'] for d in res]

    def get_table_fkeys(self, tbl):
        qry= ('pragma foreign_key_list(%s)' % tbl)
        res= self._gquery(qry, {})
        return [(d['to'], d['table'], d['from']) for d in res]

    def get_table_fields(self, tbl):
        qry= ('pragma table_info(%s)' % tbl)
        res= self._gquery(qry, {})
        return [d['name'] for d in res]

    def get_table_pkeys(self, tbl):
        qry= ('pragma table_info(%s)' % tbl)
        res= self._gquery(qry, {})
        return [d['name'] for d in res if d['pk']]



def init_sqlite(vars):
    try:
        _SqliteInfo.dbfile= vars['DB_FILE']
    except:
        Speaker.log_int_raise('Variable %s not specified!', ERR_COL('DB_FILE'))
    



from mother.mothers import *

class ClsLifes(DbMother):
    table_name= 'lifes'
    def __init__(self, store= {}, flag= MO_NOA, session= None):
        DbMother.__init__(self, store, flag, session)

class ClsMoon(DbMother):
    table_name= 'moons_info'
    def __init__(self, store= {}, flag= MO_NOA, session= None):
        DbMother.__init__(self, store, flag, session)

class ClsPlanets(DbMother, MotherManager):
    table_name= 'planets'
    def __init__(self, store= {}, flag= MO_NOA, session= None):
        self.initRelationManager([ClsLifes])
        self.initChildManager([ClsMoon])
        DbMother.__init__(self, store, flag, session)

class ClsStars(DbMother, MotherManager):
    table_name= 'stars'
    def __init__(self, store= {}, flag= MO_NOA, session= None):
        self.initChildManager([ClsPlanets])
        DbMother.__init__(self, store, flag, session)



# Copyright (c) 2007, Federico Tomassini aka efphe (effetom AT gmail DOT com)
# All rights reserved.
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the University of California, Berkeley nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS

""" 
Mother is a python module that hides SQL syntax and gives 
you a set of intelligent classes and methods. With `intelligent` 
we mean the capability of self-adaption, understanding various 
situations.
Mother could be considered as a Object Relational Mapper with a 
strong introspection. 
In fact, configuration files, tables, fields and keys declarations 
are not needed, because Mother knows the database structure herself.
Mother works with PostgreSQL and SQlite.
BSD License.
"""

__all__     = ['speaker', 'abdbda', 'commons', 'eccez', 'mothers']
__version__ = '0.6 0'
__author__  = 'Federico Tomassini aka efphe'
__contact__ = 'Report Bugs and Ideas to users@dbmother.org '\
              '(need to subscribe) or to efphe@dbmother.org'
__website__ = 'www.dbmother.org'



import psycopg2
from psycopg2.extras import DictCursor as _cursor_factory
from mother.speaker import Speaker
from mother.commons import ERR_COL

from mother.eccez import QueryError, BrokenConnection

class _PostgresInfo:
    dbuser= None
    dbpasswd= None
    dbname= None
    dbport= 5432
    dbhost= 'localhost'

    @staticmethod
    def _connect_str():

        dbhost= _PostgresInfo.dbhost
        dbname= _PostgresInfo.dbname
        dbpasswd= _PostgresInfo.dbpasswd
        dbuser= _PostgresInfo.dbuser

        # Unix Sockets?
        if not dbhost:
            return "dbname=%s user=%s password=%s" % (dbname, dbuser, dbpasswd)

        dbport= _PostgresInfo.dbport
    
        return ( "dbname=%s user=%s host=%s password=%s port=%d" 
            % (dbname, dbuser, dbhost, dbpasswd, dbport) )

class MotherPostgres:
    
    def __init__(self):
        Speaker.log_insane('Initializing connection...')
        self._connect()

    def _connect(self):
        from psycopg2.extras import DictCursor as _cf
        try:
            s= _PostgresInfo._connect_str()
            self.connection= psycopg2.connect(s)
            self.cursor= self.connection.cursor(cursor_factory= _cf)
        except Exception, ss:
            Speaker.log_raise('Unable to establish a connection '
                    'with the database: %s', ERR_COL(ss), 
                    BrokenConnection)

    def _rollback(self):
        self.connection.rollback()

    def _commit(self):
        self.connection.commit()

    def _close(self):
        self.connection.close()

    def _lastrowid(self):
        return self.cursor.lastrowid

    def _execute(self, q, d):
        d= d or None
        try:
            self.cursor.execute(q, d)
        except psycopg2.OperationalError:
            Speaker.log_raise('Connection is broken...', BrokenConnection)
        except Exception, ss:
            Speaker.log_raise(ERR_COL(ss), QueryError)


    def _gquery(self, q, d):
        self._execute(q, d)
        c= self.cursor
        res= c.fetchall()

        # we need to conver dictRow to dict
        lres= []
        for r in res:
            d= {}
            d.update(r)
            lres.append(d)

        return lres
    
    def _qquery(self, q, d):

        self._execute(q, d)

    def _mogrify(self, q, d):
        return self.cursor.mogrify(q, d)

    def get_tables(self):
        qry= ("SELECT table_name from information_schema.tables "
              "WHERE table_schema='public'")
        res= self._gquery(qry, {})
        return [d['table_name'] for d in res]

    def get_table_fkeys(self, tbl):
        qry= (
            "SELECT pt.tgargs, pt.tgnargs, pt.tgdeferrable, pt.tginitdeferred,"
            "pg_proc.proname, pg_proc_1.proname FROM pg_class pc,"
            "pg_proc pg_proc, pg_proc pg_proc_1, pg_trigger pg_trigger,"
            "pg_trigger pg_trigger_1, pg_proc pp, pg_trigger pt "
            "WHERE  pt.tgrelid = pc.oid AND pp.oid = pt.tgfoid "
            "AND pg_trigger.tgconstrrelid = pc.oid "
            "AND pg_proc.oid = pg_trigger.tgfoid "
            "AND pg_trigger_1.tgfoid = pg_proc_1.oid "
            "AND pg_trigger_1.tgconstrrelid = pc.oid "
            "AND ((pc.relname= '%s') "
            "AND (pp.proname LIKE '%%ins') "
            "AND (pg_proc.proname LIKE '%%upd') "
            "AND (pg_proc_1.proname LIKE '%%del') "
            "AND (pg_trigger.tgrelid=pt.tgconstrrelid)) " % tbl)

        mykeys=[]
        result= []
        res= self._gquery(qry, {})

        for d in res:
            buf=d['tgargs']
            s= str(buf)
            l= s.split("\x00")
            mykey= l[4]
            if mykey in mykeys:
                continue
            tbl= l[2]
            key= l[5]
            result.append((mykey, tbl, key))
            mykeys.append(mykey)
        return result 

    def get_table_fields(self, tbl):
        qry= ("SELECT column_name from information_schema.columns "
                "WHERE table_name='%s'" % tbl)
        res= self._gquery(qry, {})
        return [d['column_name'] for d in res]

    def get_table_pkeys(self, tbl):
        qry= ("SELECT column_name from information_schema.key_column_usage "
            "JOIN information_schema.table_constraints on "
            "information_schema.key_column_usage.constraint_name="
            "information_schema.table_constraints.constraint_name "
            "WHERE information_schema.key_column_usage.table_name='%s' and "
            "information_schema.table_constraints.constraint_type='PRIMARY KEY'" % tbl)
        res= self._gquery(qry, {})
        return [d['column_name'] for d in res]



def init_postgres(vars):
    try:
        _PostgresInfo.dbuser= vars['DB_USER']
    except:
        Speaker.log_int_raise('Variable %s not specified!', ERR_COL('DB_USER'))
    try:
        _PostgresInfo.dbname= vars['DB_NAME']
    except:
        Speaker.log_int_raise('Variable %s not specified!', ERR_COL('DB_NAME'))
    
    _PostgresInfo.dbpasswd= vars.get('DB_PASSWD', '')
    _PostgresInfo.dbhost= vars.get('DB_HOST', 'localhost')
    _PostgresInfo.dbport= vars.get('DB_PORT', 5432)




#!/usr/bin/env python

from distutils.core import setup

__version__ = '0.6.0'

mo_desc="""Mother is a Python Orm oriented to introspection and self autoadaption."""

mo_long_desc="""\
Mother is a python module that hides SQL syntax and gives 
you a set of intelligent classes and methods. With `intelligent` 
we mean the capability of self-adaption, understanding various 
situations.
Mother could be considered as a Object Relational Mapper with a 
strong introspection. 
In fact, configuration files, tables, fields and keys declarations 
are not needed, because Mother knows the database structure herself.
Mother works with PostgreSQL and SQlite.
BSD License.
"""

mo_classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Database',
        'Topic :: Software Development :: Libraries :: Python Modules'
]

setup(name='Mother',
    version= __version__,
    author='Federico Tomassini aka efphe',
    author_email='efphe@dbmother.org',
    maintainer= 'Federico Tomassini aka efphe',
    maintainer_email='efphe@dbmother.org',
    url='http://www.dbmother.org',
    packages=['mother', 'mother.plugins'],
    scripts= ['mothermapper'],
    classifiers= mo_classifiers,
    description= mo_desc,
    long_description= mo_long_desc,
    license= "BSD",
    platforms = ["any"],
)




from speaker import Speaker
from commons import OKI_COL, INF_COL, ERR_COL
from eccez import QueryError, ConnectionError, BrokenConnection, InvalidFilter

"""
The Mother DB engine.
"""
DB_ENGINE_PGRES  = 1
DB_ENGINE_SQLITE = 2

DB_POOL_LIMITED = 1
DB_POOL_ELASTIC = 2
DB_POOL_GROWING = 3

_J= ', '.join
_A= ' AND '.join

SQL_DEFAULT = "MOTHER_SQL_DEFAULT"
SQL_NULL    =  None
SQL_TRUE    = "True"
SQL_FALSE   = "False"

class _DbInfo:
    iface_builder= None
    arg_format= None
    db_initialized= False

class MoFilter:

    def __init__(self, filter= None, tbl= None, store= None):
        self.locals= {}
        self.strfilter = []
        self.pre_filter = []
        self.post_filter = []

        #if filter and store and tbl:
        self.add_filter(filter, tbl, store)

        #elif filter or store or tbl:
            #Speaker.log_raise('Invalid Filter Args: %s, %s, %s', 
                    #ERR_COL(filter), ERR_COL(store), ERR_COL(tbl), 
                    #InvalidFilter)
        #else:
            #pass

    def _arg_format(self, k):
        return _DbInfo.arg_format % k 

    def return_filter(self):
        pre= self.pre_filter
        post= self.post_filter

        if len(pre) > 1 or len(post) > 1:
            Speaker.log_int_raise('Cannot handle Filter: pre, post= %s, %s', 
                    ERR_COL(pre), ERR_COL(post))

        if pre: f= pre[0]
        else:   f= '' 

        strfilter= self.strfilter
        if len(strfilter):
            strfilter= _A(strfilter)
            strfilter= ('WHERE' in strfilter or 'WHERE' in f )  \
                               and 'AND %s' % strfilter         \
                               or  'WHERE %s' % strfilter
            f= '%s %s' % (f, strfilter)

        if post:
            f= '%s %s' % (f, post[0])

        return f, self.locals

    def add_store(self, d):
        self.locals.update(d)

    def old_return_filter(self):
        s, d= self._return_filter()
        if self.post_filter:
            s= '%s %s' % (s, _J(self.post_filter))
        return s, d

    def return_ftrqry(self, qry):
        f, loc= self.return_filter()
        return '%s %s' % (qry, f), loc

    def add_post(self, filter):
        # XXX accepts also dicts?
        self.post_filter.append(filter)

    def add_filter(self, filter, tbl= None, store= None):

        if not filter:
            return

        def t(tp):
            return isinstance(filter, tp)
            
        if store: 
            self.locals.update(store)

        if t(str):

            if 'JOIN' in filter:
                self.pre_filter.append(filter)
            else:
                self.strfilter.append(filter)
            return

        afrm= self._arg_format

        res= []
        if t(dict):
            for k, v in filter.iteritems():
                if v== SQL_DEFAULT:
                    if tbl:
                        res.append('%s.%s= DEFAULT' % (tbl, k))
                    else:
                        res.append('%s= DEFAULT' %  k)

                elif v == SQL_NULL:
                    if tbl:
                        res.append('%s.%s IS NULL' % (tbl, k))
                    else:
                        res.append('%s IS NULL' %  k)
                else:
                    if tbl:
                        res.append('%s.%s= %s' % (tbl, k, afrm(k)))
                    else:
                        res.append('%s= %s' % (k, afrm(k)))

            self.strfilter.append(_A(res))
            self.locals.update(filter)
            return

        # ToDo: just assume that filter is iterable?
        if t(list) or t(set) or t(frozenset):
            if not store:
                Speaker.log_raise('Invalid Filter: store not provided '
                                  'for iterator filter.', InvalidFilter)
            for k in filter:
                try:
                    v= store[k]
                    self.locals[k]= v
                except:
                    Speaker.log_raise('Invalid Filter: key %s not in store', 
                            ERR_COL(k), InvalidFilter)

                if v== SQL_DEFAULT:
                    if tbl:
                        res.append('%s.%s= DEFAULT' % (tbl, k))
                    else:
                        res.append('%s= DEFAULT' %  k)
                elif v == SQL_DEFAULT:
                    if tbl:
                        res.append('%s.%s IS NULL' % (tbl, k))
                    else:
                        res.append('%s IS NULL' %  k)
                else:
                    if tbl:
                        res.append('%s.%s= %s' % (tbl, k, afrm(k)))
                    else:
                        res.append('%s= %s' % (k, afrm(k)))

            self.strfilter.append(_A(res))
            return
        
        Speaker.log_raise('Invalid Filter: type %s not allowed.', 
                ERR_COL(type(filter)), InvalidFilter)


class DbOne(Speaker):

    trans_level= 0
    _db_initialized= False
    _iface_instance= None
    _iface_attrs = [
            '_connect',
            '_rollback',
            '_commit',
            '_gquery',
            '_qquery'
            ]

    # Iface methods
    # _connect()  
    # _rollback()
    # _commit()
    # _get_query(qry, filter)
    # _quiet_query(qry, filter)

    @staticmethod
    def _import_iface():
        iface= DbOne._iface_instance= _DbInfo.iface_builder()
        for attr in DbOne._iface_attrs:
            setattr(DbOne, attr, getattr(iface, attr))
        if hasattr(DbOne._iface_instance, '_mogrify'):
            DbOne.mogrify= staticmethod(DbOne._iface_instance._mogrify)

    @staticmethod
    def beginTrans():
        DbOne.trans_level+= 1
        Speaker.log_debug(
                'Incremented transaction level: %s', 
                INF_COL(DbOne.trans_level))

    @staticmethod
    def rollback():
        if not DbOne.trans_level:
            Speaker.log_warning(
                    "Nothing to rollback: "
                    "nested rollback?")
            return

        DbOne._rollback()
        DbOne.trans_level= 0

        return

    @staticmethod
    def commit():
        lvl= DbOne.trans_level
        if not lvl:
            Speaker.log_warning(
                    "Nothing to commit: "
                    "nested commit?")
            return

        if lvl== 1:
            DbOne._commit()
            DbOne.trans_level-= 1
            Speaker.log_debug(
                    "Queries committed.")
            return

        DbOne.trans_level-= 1
        Speaker.log_debug(
                "Decremented transaction: now %s",
                INF_COL(DbOne.trans_level))
        return

    @staticmethod
    def lastrowid():
        return DbOne._iface_instance._lastrowid()

    @staticmethod
    def _do_query(s, filter= None, result= True):

        execattr= result and DbOne._gquery or DbOne._qquery

        if not filter:
            d= {}
        elif isinstance(filter, MoFilter):
            s, d= filter.return_ftrqry(s)
        elif isinstance(filter, dict):
            d= filter
        else:
            Speaker.log_int_raise('Invalid Filter Type: %s', 
                    ERR_COL(type(filter)))

        # logging info...
        mogrify= getattr(DbOne._iface_instance, '_mogrify', None)
        if mogrify is not None:
            Speaker.log_info("QSQL- %s", mogrify(s, d))
        else:
            Speaker.log_info("QSQL- %s, Filter= %s" % (s, d))

        try:
            return execattr(s, d)

        except BrokenConnection, e:

            Speaker.log_info('Connection to DB seems broken, '
                               'now reconnecting...')
            try:
                DbOne._connect()
            except:
                Speaker.log_raise('Cannot re-establish connection', BrokenConnection)

            # if we are inside a trans, we have to signal 
            # the broken transaction -> exception.
            # otherwise, the query is tried once more
            if DbOne.trans_level:
                Speaker.log_raise('Connection re-established: transaction is lost.', 
                        ConnectionError)
        
            return execattr(s, d)

        except QueryError, e:
            DbOne._rollback()
            Speaker.log_raise('Rollbacked queries: %s.', ERR_COL(e), QueryError)


    @staticmethod
    def _get_query(s, filter= None):

        res= DbOne._do_query(s, filter, True)
        if not DbOne.trans_level:
            DbOne._commit()
        return res

    @staticmethod
    def _quiet_query(s, filter= None):

        DbOne._do_query(s, filter, False)
        if not DbOne.trans_level:
            DbOne._commit()

    @staticmethod
    def oc_query(s, filter= None):
        """ One Commit Query: no returns."""
        DbOne._quiet_query(s, filter)

    @staticmethod
    def mr_query(s, filter= None):
        """ Multiple Records Query: return a dict list."""
        return DbOne._get_query(s, filter)

    @staticmethod
    def or_query(s, filter= None):
        """ One Record Query: returns a dict."""

        res= DbOne.mr_query(s, filter)
        
        if len(res)<> 1:
            Speaker.log_raise(
                    "Query returned %s records instead of 1." % 
                    ERR_COL(len(res)), QueryError)

        return res[0]

    @staticmethod
    def ov_query(s, filter):
        """ One Value Query: returns a unique value."""
        
        res= DbOne.or_query(s, filter)

        res= res.values()
        if len(res)<> 1:
            Speaker.log_raise(
                    "Query returned %s values instead of 1." % 
                    ERR_COL(len(res)), QueryError)

        return res[0]

class DbFly(Speaker):

    # Iface methods
    # _connect()  
    # _rollback()
    # _commit()
    # _get_query(qry, filter)
    # _quiet_query(qry, filter)
    # _close()

    _default_session= 'DbFly'

    _iface_attrs = [
            '_connect',
            '_rollback',
            '_commit',
            '_gquery',
            '_qquery',
            '_close'
            ]

    exported_methods= [
            'lastrowid',
            'oc_query',
            'ov_query',
            'mr_query',
            'or_query',
            'beginTrans',
            'commit',
            'rollback',
            'endSession',
            ]

    def __init__(self, name= None):

        self.session_name= name or self._default_session
        self._import_iface()
        self._connect()

    def _import_iface(self):
        iface= self._iface_instance= _DbInfo.iface_builder()
        for attr in self._iface_attrs:
            setattr(self, attr, getattr(iface, attr))
        if hasattr(self._iface_instance, '_mogrify'):
            self.mogrify= DbOne._iface_instance._mogrify

    def _export_iface(self, where):
        for attr in self.exported_methods:
            setattr(where, attr, getattr(self, attr))

    def lastrowid(self):
        return self._iface_instance._lastrowid()

    def beginTrans(self):
        self.log_noise('"Transactions is the default '
                    "inside Sessions (session= %s).",
                    INF_COL(self.session_name))

    def rollback(self):
        self.log_debug("Rollbacking queries for Session %s", 
                self.session_name)
        self._rollback()

    def commit(self):
        try:
            self._commit()
            self.log_debug("Queries Committed for Session %s.", 
                    OKI_COL(self.session_name))
            return 0
        except Exception, ss: 
            return ss

    def endSession(self):
        self.log_info("Terminating Session %s",
                OKI_COL(self.session_name))

        if not hasattr(self, '_pool_queue'):
            self.log_insane("No Pool: Session %s will be closed.", 
                    OKI_COL(self.session_name))
            self.commit()
            self._close()
            return

        res= self.commit()

        if res <> 0:
            MotherPool.discard(self)
            self.log_error("Session %s, unable to commit queries(): "
                        "%s. Broken session removd from the pool.", 
                        ERR_COL(self.session_name), ERR_COL(res))

        MotherPool.backHome(self)

    def _do_query(self, s, filter= None, result= True):

        execattr= result and self._gquery or self._qquery

        if not filter:
            d= {}
        elif isinstance(filter, MoFilter):
            s, d= filter.return_ftrqry(s)
        elif isinstance(filter, dict):
            d= filter
        else:
            self.log_int_raise('Invalid Filter Type: %s', 
                    ERR_COL(type(filter)))

        mogrify= getattr(self._iface_instance, '_mogrify', None)
        if mogrify is not None:
            self.log_info("%s: QSQL- %s", INF_COL(self.session_name), mogrify(s, d))
        else:
            self.log_info("%s: QSQL- %s, Filter= %s" , INF_COL(self.session_name), s, d)

        try:
            return execattr(s, d)

        except BrokenConnection, e:

            self.log_info('Connection to DB seems broken, '
                               'now reconnecting...')
            try:
                self._connect()
            except:
                self.log_raise('Cannot re-establish connection', BrokenConnection)

            # if we are inside a trans, we have to signal 
            # the broken transaction -> exception.
            # otherwise, the query is tried once more
            if self.trans_level:
                self.log_raise('Connection re-established: transaction is lost.', 
                        ConnectionError)
        
            return execattr(s, d)

        except QueryError, e:
            self._rollback()
            self.log_raise('Rollbacked queries: %s.', ERR_COL(e), QueryError)


    def _get_query(self, s, filter= None):

        return self._do_query(s, filter, True)

    def _quiet_query(self, s, filter= None):

        self._do_query(s, filter, False)

    def oc_query(self, s, filter= None):
        """ One Commit Query: no returns."""
        self._quiet_query(s, filter)

    def mr_query(self, s, filter= None):
        """ Multiple Records Query: return a dict list."""
        return self._get_query(s, filter)

    def or_query(self, s, filter= None):
        """ One Record Query: returns a dict."""

        res= self.mr_query(s, filter)
        
        if len(res)<> 1:
            self.log_raise(
                    "Query returned %s records instead of 1." % 
                    ERR_COL(len(res)), QueryError)

        return res[0]

    def ov_query(self, s, filter= None):
        """ One Value Query: returns a unique value."""
        
        res= DbOne.or_query(s, filter)

        res= res.values()
        if len(res)<> 1:
            self.log_raise(
                    "Query returned %s values instead of 1." % 
                    ERR_COL(len(res)), QueryError)

        return res[0]


class MotherPool:

    from Queue import Empty as _empty_queue

    _pool_initialized= False

    _pool_max= 10
    _pool_min= 4
    _pool_timeout= 15
    _pool_current= 0
    _pool_orphans= {}
    _pool_queue= None
    _pool_type= DB_POOL_LIMITED
    _pool_calm= False

    import threading

    _pool_current_mutex= threading.Lock()
    _pool_get_mutex= threading.Lock()
    _pool_orphans_mutex= threading.Lock()

    del threading

    @staticmethod
    def _pool_type_str(ptype):

        if ptype == DB_POOL_LIMITED:
            return 'Limited'
        elif ptype == DB_POOL_ELASTIC: 
            return 'Elastic'
        else:
            return 'Growing'

    @staticmethod
    def _add_orphan(sname, db):
        m= MotherPool._pool_orphans_mutex
        m.acquire()
        MotherPool._pool_orphans[id(db)]= sname
        m.release()

    @staticmethod
    def _del_orphan(db):
        m= MotherPool._pool_orphans_mutex
        m.acquire()
        MotherPool._pool_orphans.pop(id(db))
        m.release()

    @staticmethod
    def _full():

        ptype= MotherPool._pool_type

        if ptype in [DB_POOL_ELASTIC, DB_POOL_GROWING]:
            return False

        # Db Pool is Limited
        m= MotherPool._pool_current_mutex
        m.acquire()
        res= MotherPool._pool_current== MotherPool._pool_max
        m.release()
        return res

    @staticmethod
    def status():
        """ status() -> (pool_type, available, total, min, max, orphaned)
        """

        mc= MotherPool._pool_current_mutex
        mg= MotherPool._pool_get_mutex
        mo= MotherPool._pool_orphans_mutex
        mc.acquire()
        mg.acquire()
        mo.acquire()
        total= MotherPool._pool_current
        available= MotherPool._pool_queue.qsize()
        orphaned= MotherPool._pool_orphans.values()
        mo.release()
        mg.release()
        mc.release()
        min= MotherPool._pool_min
        max= MotherPool._pool_max

        ptype= MotherPool._pool_type
        sptype= MotherPool._pool_type_str(ptype)

        return (sptype, available, total, min, max, orphaned)

    @staticmethod
    def _add_conn(n=1):
        """ Never call me directly: use _addConnection!"""

        cur= MotherPool._pool_current
        max= MotherPool._pool_max 

        Speaker.log_info(
                "Adding %s new connection(s) (max= %s, cur= %s)",
                OKI_COL(n), OKI_COL(max), OKI_COL(cur))

        for i in xrange(n):
            p= DbFly()
            q= MotherPool._pool_queue
            p._pool_queue= q
            q.put(p)

        MotherPool._pool_current+= n

        return n

    ##
    # Note that this function could be called by:
    #  init_pool() -> no problem with mutexes
    #  newSession() -> no problem with mutexes, coz
    #                  lock is acquired.
    ##
    @staticmethod
    def _addConnection(n=1):
        """ 
        Don't call me directly, use newSession() instead!
        """
        m= MotherPool._pool_current_mutex
        m.acquire()
        MotherPool._add_conn(n)
        m.release()


    @staticmethod
    def _init_pool(n= None, pg_conn= None):

        n= n or MotherPool._pool_min

        Speaker.log_info("Initializing connection Pool ...")
        from Queue import Queue
        MotherPool._pool_queue= Queue()
        MotherPool._addConnection(n)

        MotherPool._pool_initialized= True

    @staticmethod
    def _get_session():
        """ Never call me directly: use newSession!"""

        db_pool= MotherPool._pool_queue

        try:
            db= db_pool.get_nowait()
            return db
        except MotherPool._empty_queue:
            pass

        full= MotherPool._full()
        calm= MotherPool._pool_calm

        # Wait or create immediately a new connection?
        if calm or full:
            
            # wait.
            ptimeout= MotherPool._pool_timeout
            Speaker.log_info(
                "MotherPool: waiting for a free connection "
                "(timeout= %s) ...", INF_COL(ptimeout))

            # If full, wait and hope.
            if full:
                return db_pool.get(True, ptimeout)

            # If calm, wait only
            try:
                return db_pool.get(True, ptimeout)
            except MotherPool._empty_queue:
                pass
        
        # Ok, here the pool is not full.
        # Moreover, we have already waited.
        MotherPool._addConnection()
        return db_pool.get_nowait()

    @staticmethod
    def newSession(name= None):

        Speaker.log_info("Initializing session %s", INF_COL(name))

        if not MotherPool._pool_initialized:
            return DbFly(name)

        m= MotherPool._pool_get_mutex
        m.acquire()

        try:
            session= MotherPool._get_session()

        except Exception, ss:
            m.release()
            Speaker.log_int_raise(
                    "Cannot retrieve Session from Pool (%s). FATAL.", 
                    ERR_COL(ss))

        sname= name or DbFly._default_session

        MotherPool._add_orphan(sname, session)

        m.release()
        session.session_name= sname 

        return session

    @staticmethod
    def backHome(dbfly):

        q= MotherPool._pool_queue
        m= MotherPool._pool_current_mutex
        sname= dbfly.session_name

        m.acquire()

        if MotherPool._pool_type == DB_POOL_ELASTIC and \
           MotherPool._pool_current > MotherPool._pool_min:
                MotherPool._ns_discard(dbfly)
                Speaker.log_info("Elastic Pool: session %s "
                                 "closed and removed.", OKI_COL(sname))
                m.release()
                return

        try:
            q.put_nowait(dbfly)
        except Exception, ss:
            Speaker.log_warning(
                    "Removing connection %s from Pool: %s.", 
                    ERR_COL(dbfly.session_name), ERR_COL(ss))
            MotherPool._ns_discard(dbfly)
            m.release()
            return

        MotherPool._del_orphan(dbfly)
        Speaker.log_info('Session %s back to the Pool.', OKI_COL(sname))
        m.release()
        return

    @staticmethod
    def _ns_discard(dbfly):

        MotherPool._pool_current-= 1
        MotherPool._del_orphan(dbfly)
        try:
            dbfly._close()
        except Exception, ss:
            Speaker.log_error("Unable to close connection: %s", ERR_COL(ss))

    @staticmethod
    def discard(dbfly):

        m= MotherPool._pool_current_mutex

        m.acquire()
        MotherPool._ns_discard(dbfly)
        m.release()
        Speaker.log_warning("Connection %s dropped from Pool.", 
                ERR_COL(dbfly.session_name))



def init_abdbda(conf, forced= {}):

    if _DbInfo.db_initialized:
        Speaker.log_warning('AbDbDa already initialized.')
        return

    err= None
    if not isinstance(conf, dict):
        import speaker
        loc= {}
        names_dict= speaker.__dict__.copy()
        names_dict.update(globals())
        try:
            execfile(conf, names_dict, loc)
        except Exception, ss:
            err= "Unable to read Mother configuration "  \
                 "file %s: %s" % (ERR_COL(conf), ss)
    else:
        loc= conf

    loc.update(forced)

    if hasattr(Speaker, '_spkr_initialized'):
        from speaker import init_speaker
        init_speaker(loc)

    # now that speaker is up, error is printable
    if err:
        Speaker.log_int_raise(err)

    # set the right db engine
    db_engine= loc.get('DB_ENGINE', DB_ENGINE_PGRES)
    if db_engine == DB_ENGINE_PGRES:
        from mother.postgres import MotherPostgres as _builder, \
                                     init_postgres as init_engine
        _DbInfo.arg_format= '%%(%s)s'
    elif db_engine == DB_ENGINE_SQLITE:
        from mother.sqlite import MotherSqlite as _builder, \
                                   init_sqlite as init_engine
        _DbInfo.arg_format= ':%s'

    init_engine(loc)
    _DbInfo.iface_builder= _builder

    pool= loc.get('DB_POOL', False)
    db_one= loc.get('DB_PERSISTENT_ONE', False)

    if not pool or db_one:
        DbOne._import_iface()
        DbOne._connect()
        DbOne._db_initialized= True

    if not pool:
        _DbInfo.db_initialized= True
        return

    pool_to= loc.get('DB_POOL_TIMEOUT', MotherPool._pool_timeout)
    pool_min= loc.get('DB_POOL_MIN', MotherPool._pool_min)
    pool_max= loc.get('DB_POOL_MAX', MotherPool._pool_max)
    pool_calm= loc.get('DB_POOL_PATIENT', False)
    pool_type= loc.get('DB_POOL_TYPE', MotherPool._pool_type)

    if pool_type == DB_POOL_LIMITED and pool_min> pool_max:
        Speaker.log_int_raise("Invalid configuration file: %s",
                ERR_COL("DB_POOL_MIN > DB_POOL_MAX !!!"))

    MotherPool._pool_min= pool_min
    if pool_type in  [DB_POOL_ELASTIC, DB_POOL_GROWING]:
        MotherPool._pool_max= -1
    else:
        MotherPool._pool_max= pool_max

    MotherPool._pool_timeout= pool_to
    MotherPool._pool_type= pool_type
    MotherPool._pool_calm= pool_calm

    MotherPool._init_pool()
    _DbInfo.db_initialized= True

    return


